//
//  3DTouchAppDelegateExtension.swift
//  CollectMe
//
//  Created by Oscar Fernandez Ardanuy on 26/05/16.
//  Copyright © 2016 dorhus. All rights reserved.
//

import UIKit

extension AppDelegate {
 
    // MARK: Shortcut actions
    
    func application(application: UIApplication, performActionForShortcutItem shortcutItem: UIApplicationShortcutItem, completionHandler: (Bool) -> Void) {
        // Handle quick actions
        completionHandler(handleQuickAction(shortcutItem))
    }
    
    enum Shortcut: String {
        case addNewItem3dAction = "AddNewItem3dAction"
        case addWantedItem3dAction = "AddWantedItem3dAction"
    }
    
    func handleQuickAction(shortcutItem: UIApplicationShortcutItem) -> Bool {
        
        var quickActionHandled = false
        let type = shortcutItem.type.componentsSeparatedByString(".").last!
        if let shortcutType = Shortcut.init(rawValue: type) {
            switch shortcutType {
            case .addNewItem3dAction:
                navigateToNewItemView(MyTabBarOrigin.Search)
                quickActionHandled = true
            
            case .addWantedItem3dAction:
                navigateToNewItemView(MyTabBarOrigin.Wanted)
                quickActionHandled = true
                
            }
        }
        
        return quickActionHandled
    }
    
    private func navigateToNewItemView(origin: MyTabBarOrigin) {
        let storyboard = UIStoryboard(name: "Main", bundle: nil)
        var viewControllers: [UIViewController] = []
        
        // select tab
        var tabBar : UITabBarController
        
        if window?.rootViewController?.presentedViewController == nil {
            tabBar = storyboard.instantiateViewControllerWithIdentifier("TabBarView") as! UITabBarController
            tabBar.selectedIndex = origin.rawValue
            
            window?.rootViewController?.presentViewController(tabBar, animated: true, completion: nil)
        } else {
            tabBar = window!.rootViewController!.presentedViewController as! UITabBarController
            tabBar.selectedIndex = origin.rawValue
        }
        
        let searchNavigation : UINavigationController = tabBar.viewControllers?[origin.rawValue] as! UINavigationController
        
        // tabBar.viewControllers?[MyTabBarOrigin.Search.rawValue].childViewControllers
        let itemsListView : ItemsListViewController = storyboard.instantiateViewControllerWithIdentifier("ItemsListView") as! ItemsListViewController
        let newItemView : ItemsNewItemTableViewController = storyboard.instantiateViewControllerWithIdentifier("NewItemView") as! ItemsNewItemTableViewController
        newItemView.newItemPage = true
        
        viewControllers.append(itemsListView)
        viewControllers.append(newItemView)
        
        searchNavigation.setViewControllers(viewControllers, animated: true)
        //searchNavigation.pushViewController(viewControllers[0], animated: true)
        //searchNavigation.pushViewController(viewControllers[1], animated: true)
    }
    
}