//
//  CarouselPictures.swift
//  CollectMe
//
//  Created by Oscar Fernandez Ardanuy on 09/05/16.
//  Copyright © 2016 dorhus. All rights reserved.
//

import Foundation
import UIKit

class CarouselPictures {
    // pictures from data store
    private var picturesArray : [PicturesThumbnail]
    // new pictures added
    private var newPicturesArray : [PictureWithThumbnail]
    // pictures from data store deleted
    private var deletedPicturesArray : [PicturesThumbnail]
    // all thumbnails prepared for carousel
    private var imageArray : [UIImage]
    
    init() {
        self.picturesArray = []
        self.newPicturesArray = []
        self.imageArray = []
        self.deletedPicturesArray = []
    }
    
    init(picturesArray : [PicturesThumbnail]) {
        self.picturesArray = []
        self.newPicturesArray = []
        self.imageArray = []
        self.deletedPicturesArray = []
        
        for picture in picturesArray {
            self.addNewPictureFrom(pictureWithThumbnail: picture)
        }
    }
    
    init(picturesArray : [PicturesThumbnail], imageArray : [UIImage]) {
        self.picturesArray = picturesArray
        self.newPicturesArray = []
        self.imageArray = imageArray
        self.deletedPicturesArray = []
    }
    
    func getImageArray() -> [UIImage] {
        return self.imageArray
    }
    
    func getNewPicturesArray() -> [PictureWithThumbnail] {
        return self.newPicturesArray
    }
    
    func getDeletedOldPicturesArray() -> [PicturesThumbnail] {
        return self.deletedPicturesArray
    }
    
    func getIndexOf(image: UIImage) -> Int? {
        if let index = self.imageArray.indexOf(image) {
            return index
        }
        return nil
    }
    
    func getFullResOf(image: UIImage) -> UIImage? {
        let imageFullResData: NSData
        
        if let index = getIndexOf(image) {
            // pictures from store data
            if index + 1 <= self.picturesArray.count {
                // exists? is possible that one parallel process has removed the image
                if (self.picturesArray[index].fullRes != nil) {
                    imageFullResData = self.picturesArray[index].fullRes!.image_data!
                // image deleted from other place
                } else {
                    self.picturesArray.removeAtIndex(index)
                    self.imageArray.removeAtIndex(index)
                    return nil
                }
                // new pictures
            } else {
                imageFullResData = self.newPicturesArray[index - self.picturesArray.count].picture
            }
            
            return Utils.NSDataToUIImage(imageFullResData)
        }
        
        return nil
    }
    
    func addNewPictureFrom(original image: UIImage) {
        // use date as unique id
        let date : Double = NSDate().timeIntervalSince1970
        guard let imageData = UIImageJPEGRepresentation(image, 1) else {
            //failed conversion
            //print("Error converting JPG")
            return
        }
        
        // scale image
        let thumbSize = CGSize(width: 400, height: 400)
        let thumbnail = image.scale(toSize: thumbSize)
        
        guard let thumbnailData = UIImageJPEGRepresentation(thumbnail, 0.7) else {
            //failed conversion
            //print("Error converting thumbnail JPG")
            return
        }
        
        let newPicture:PictureWithThumbnail = PictureWithThumbnail(picture: imageData, thumbnail: thumbnailData, timestamp: date, best: false)
        self.newPicturesArray.append(newPicture)
        self.imageArray.append(thumbnail)
    }
    
    func addNewPictureFrom(pictureWithThumbnail picture: PicturesThumbnail) {
        if let smallImage = Utils.NSDataToUIImage(picture.image_data!) {
        //if let smallImage = UIImage(data: picture.image_data!, scale:1.0) {
            self.picturesArray.append(picture)
            self.imageArray.append(smallImage)
        } else {
            //print("Error converting thumbnail NSData to Image")
        }
    }
    
    func removePicture(withThumbnail thumbnail: UIImage) {
        if let index = self.getIndexOf(thumbnail) {
            removePicture(atIndex: index)
        }
    }
    
    func removePicture(atIndex index: Int) {
        if self.imageArray.count > index {
            self.imageArray.removeAtIndex(index)
            
            // pictures from store data
            if index + 1 <= self.picturesArray.count {
                self.deletedPicturesArray.append(self.picturesArray[index])
                self.picturesArray.removeAtIndex(index)
            // new pictures
            } else {
                self.newPicturesArray.removeAtIndex(index - self.picturesArray.count)
            }
        }
    }
    
}
