//
//  CarouselPicturesINITVCExt.swift
//  CollectMe
//
//  Created by Oscar Fernandez Ardanuy on 29/04/16.
//  Copyright © 2016 dorhus. All rights reserved.
//

import Foundation
import UIKit

extension ItemsNewItemTableViewController {
    
    internal func printImagesInScrollView() {
        
        var scrollViewContentSize: CGFloat = 0;
        let yPosition: CGFloat = 0
        var xPosition: CGFloat = 10
        let imageWidth: CGFloat = 150
        let imageHeight: CGFloat = 150
        let spacer: CGFloat = 10
        
        for subview in picturesScrollView.subviews {
            subview.removeFromSuperview()
        }
        
        //for oneImage in self.picturesArray {
        for oneImage in self.picturesArray.getImageArray() {
            let myImageView: UIImageView = UIImageView()
            
            myImageView.image = oneImage
            myImageView.contentMode = UIViewContentMode.ScaleAspectFill
            myImageView.clipsToBounds = true
            myImageView.frame.size.width = imageWidth
            myImageView.frame.size.height = imageHeight
            myImageView.center = self.picturesScrollView.center
            myImageView.frame.origin.y = yPosition
            myImageView.frame.origin.x = xPosition
            
            // add to scroll
            self.picturesScrollView.addSubview(myImageView)
            
            // at tap one image
            let gestureTap = UITapGestureRecognizer(target: self, action: #selector(ItemsNewItemTableViewController.imageTapAction(_:)))
            myImageView.userInteractionEnabled = true
            myImageView.addGestureRecognizer(gestureTap)
            
            // at long press one image
            let gestureSwipe = UILongPressGestureRecognizer(target: self, action: #selector(ItemsNewItemTableViewController.imageLongPressAction(_:)))
            myImageView.userInteractionEnabled = true
            myImageView.addGestureRecognizer(gestureSwipe)
            
            // se recalculan las posiciones de los cuadros siguientes
            xPosition += imageWidth + spacer
            scrollViewContentSize += imageWidth + spacer
        }
        
        // se cambia el tamaño del scroll view
        self.picturesScrollView.contentSize = CGSize(width: scrollViewContentSize, height: imageHeight)
    }
    
    internal func imageTapAction(sender: UITapGestureRecognizer) {
        if !imagePressActionActive {
            imagePressActionActive = true
            let imageTouched = sender.view as! UIImageView
            
            if let fullResPicture = self.picturesArray.getFullResOf(imageTouched.image!) {
                self.pictureSelected = fullResPicture
                self.performSegueWithIdentifier(SEGUE_FROM_ITEM_PAGE_TO_PICTURE_PAGE, sender: self)
            } else {
                MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Image does not exist", comment: ""), view: self)
                self.printImagesInScrollView()
            }
            
            imagePressActionActive = false
        }
    }
    
    internal func imageLongPressAction(sender: UILongPressGestureRecognizer) {
        if (!infoPage && !imagePressActionActive) {
            imagePressActionActive = true
            let imageTouched = sender.view as! UIImageView
            let imageIndex = self.picturesArray.getIndexOf(imageTouched.image!)
            
            //print("Long press image nº \(imageIndex)")
            
            // Creamos alert
            let actionSheetController: UIAlertController = UIAlertController(title: nil, message: NSLocalizedString("Do you want to delete the image?", comment: ""), preferredStyle: .Alert)
            // Creamos boton de no
            let noAction: UIAlertAction = UIAlertAction(title: NSLocalizedString("Cancel", comment: ""), style: .Default) { Void in
                self.imagePressActionActive = false
            }
            // Añadimos el botón al alert
            actionSheetController.addAction(noAction)
            
            // Creamos boton de ok
            let okAction: UIAlertAction = UIAlertAction(title: NSLocalizedString("Accept", comment: ""), style: .Default) { action in
                self.picturesArray.removePicture(atIndex: imageIndex!)
                
                self.printImagesInScrollView()
                self.imagePressActionActive = false
            }
            // Añadimos el botón al alert
            actionSheetController.addAction(okAction)
            
            self.presentViewController(actionSheetController, animated: true, completion: nil)
        }
    }
}