//
//  CollectionPickerINITVCExt.swift
//  CollectMe
//
//  Created by Oscar Fernandez Ardanuy on 10/05/16.
//  Copyright © 2016 dorhus. All rights reserved.
//

import Foundation
import UIKit

extension ItemsNewItemTableViewController: UIPickerViewDataSource, UIPickerViewDelegate {

    func numberOfComponentsInPickerView(pickerView: UIPickerView) -> Int {
        return 1
    }
    
    func pickerView(pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        return self.allCollections.count + 1
    }
    
    func pickerView(pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        var title: String
        
        if (row < self.allCollections.count) {
            title = "\(self.allCollections[row].name!)"
            if !self.allCollections[row].info!.isEmpty {
                 title += "- \(self.allCollections[row].info!)"
            }
        } else {
            title = NSLocalizedString("+ new collection", comment: "")
        }
        
        return title
    }
    
    func pickerView(pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        if (row < self.allCollections.count) {
            self.thisCollection = self.allCollections[row]
        } else {
            self.thisCollection = nil
        }
        
    }
    
    internal func setUpCollectionsPicker() {
        if self.allCollections.isEmpty {
            DataManager.getAllMyCollection() { (myStoredCollections: [MyCollection]?) -> (Void) in
                if (myStoredCollections != nil) {
                    self.allCollections = myStoredCollections!
                    self.allCollections.sortInPlace({ $0.name < $1.name })
                    
                    if self.thisCollection != nil && !self.allCollections.isEmpty {
                        if let indexOfCollection:Int = self.allCollections.indexOf(self.thisCollection) {
                            self.collectionPickerView.selectRow(indexOfCollection, inComponent: 0, animated: true)
                        } else {
                            self.collectionPickerView.selectRow(self.collectionPickerView.numberOfRowsInComponent(0) - 1, inComponent: 0, animated: false)
                        }
                    } else {
                        self.collectionPickerView.selectRow(self.collectionPickerView.numberOfRowsInComponent(0) - 1, inComponent: 0, animated: false)
                    }
                    
                } else {
                    MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Collections list cannot be loaded", comment: ""), view: self)
                }
            }
        }
    }
    
    internal func showCreateNewCollectionAlert() {
        //New alert
        let alertController: UIAlertController = UIAlertController(title: nil, message: NSLocalizedString("Add new collection first", comment: ""), preferredStyle: .Alert)
        
        let newCollectionAction = UIAlertAction(title: NSLocalizedString("Add", comment: ""), style: .Default) { (_) in
            let nameTextField = alertController.textFields![0] as UITextField
            let classTextField = alertController.textFields![1] as UITextField
            
            self.nameNewCollection = nameTextField.text
            self.classNewCollection = classTextField.text
            
            ActivityIndicatorHelper.startActivityIndicator(self.activityIndicator)
            
            // Create new collection Action
            DataManager.saveAndReturnNewMyCollection(self.nameNewCollection!, info: self.classNewCollection, full: false, completion: { (newCollection:MyCollection?) -> (Void) in
                ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                
                if newCollection != nil {
                    self.thisCollection = newCollection
                    self.addOrEditItem()
                } else {
                    MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Collection cannot be created", comment: ""), view: self)
                }
            })
        }
        newCollectionAction.enabled = false
        
        let cancelAction = UIAlertAction(title: NSLocalizedString("Cancel", comment: ""), style: .Cancel) { (_) in }
        
        alertController.addTextFieldWithConfigurationHandler { (textField) in
            textField.placeholder = NSLocalizedString("Insert name", comment: "")
            
            NSNotificationCenter.defaultCenter().addObserverForName(UITextFieldTextDidChangeNotification, object: textField, queue: NSOperationQueue.mainQueue(), usingBlock: { (notification) in
                newCollectionAction.enabled = textField.text != ""
            })
        }
        
        alertController.addTextFieldWithConfigurationHandler { (textField) in
            textField.placeholder = NSLocalizedString("Insert class (Optional)", comment: "")
        }
        
        alertController.addAction(newCollectionAction)
        alertController.addAction(cancelAction)
        
        self.presentViewController(alertController, animated: true, completion: nil)
        
    }
}
