//
//  CollectionsListViewController.swift
//  CollectMe
//
//  Created by Oscar Fernandez Ardanuy on 14/04/16.
//  Copyright © 2016 dorhus. All rights reserved.
//

import UIKit
import MGSwipeTableCell

class CollectionsListViewController: UIViewController, UITableViewDataSource, MGSwipeTableCellDelegate {

    // layout objects
    @IBOutlet weak var tableView: UITableView!
    @IBAction func addNavButton(sender: AnyObject) {
        addNewCollection()
    }
    
    private var myCollectionsArray:[MyCollection] = []
    private var myCollectionToEdit:MyCollection? = nil
    
    private var activityIndicator: UIActivityIndicatorView?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Do any additional setup after loading the view.
        self.activityIndicator = ActivityIndicatorHelper.getActivityIndicator(self.view)
        setSyle()
        
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        self.loadMyCollections()
    }
    
    // MARK: - Swipe rows
    /*func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
        if editingStyle == .Delete {
            let rowNumber = indexPath.row
            let deletedCollection = myCollections[rowNumber]
            
            DataManager.deleteMyCollection(deletedCollection, completion: { (allOk: Bool) -> (Void) in
                if allOk {
                    self.myCollections.removeAtIndex(rowNumber)
                    self.tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
                } else {
                    MessageHelper.showBasicAlert("Título", message: "oye que haces premoh", view: self)
                }
            })
        }
    }*/
    
    // END: - Swipe rows
    
    
    /*
    // MARK: - Navigation
    
    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
    // Get the new view controller using segue.destinationViewController.
    // Pass the selected object to the new view controller.
    }
    */
    
    
    // MARK: UITableViewDataSource
    func tableView(tableView: UITableView,
        numberOfRowsInSection section: Int) -> Int {
            
            return myCollectionsArray.count
    }
    
    func tableView(tableView: UITableView,
        cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        
        //let cell = tableView.dequeueReusableCellWithIdentifier("MyCollectionCell", forIndexPath: indexPath) as! CollectionsListTableViewCell
        let cell = tableView.dequeueReusableCellWithIdentifier("MyCollectionCell", forIndexPath: indexPath) as! CollectionsListMGSwipeTableCell
        let rowNumber = indexPath.row

        let name = myCollectionsArray[rowNumber].name
        let info = myCollectionsArray[rowNumber].info
        let isFull = myCollectionsArray[rowNumber].isFull()
        
        cell.nameLabel.text = name
        cell.infoLabel.text = info
    
        cell.isFullImageView.image = StyleHelper.getImageForIsCompleted(isFull)
        // SWIPE functions
        let deleteButton = MGSwipeButton(title: NSLocalizedString("Delete", comment: ""), backgroundColor: UIColor.redColor())
        let editButton = MGSwipeButton(title: NSLocalizedString("Edit", comment: ""),backgroundColor: UIColor.lightGrayColor())
        
        cell.rightButtons = [deleteButton, editButton]
        cell.rightSwipeSettings.transition = MGSwipeTransition.Border
        
        cell.backgroundColor = StyleHelper.getCellColor(myCollectionsArray[rowNumber].color)
        
        cell.delegate = self
        
        return cell
    }
    // END: UITableViewDataSource
    
    
    //MARK: MGSwipeTableCellDelegate
    func swipeTableCell(cell:MGSwipeTableCell, index:Int,  direction:MGSwipeDirection, fromExpansion:Bool) -> Bool {
        return true
    }
    
    func swipeTableCell(cell: MGSwipeTableCell!, tappedButtonAtIndex index: Int, direction: MGSwipeDirection, fromExpansion: Bool) -> Bool {
        let indexPath = tableView.indexPathForCell(cell)
        
        // index is the possition of the button inside the MGSwipeTableCell
        switch index {
            case 0: // DELETE
                deleteCollection(indexPath!)
            case 1: // EDIT
                editCollection(indexPath!)
            default:
                break
        }
        
        return true
    }
    //END: MGSwipeTableCellDelegate
    
    func loadMyCollections() {
        ActivityIndicatorHelper.startActivityIndicator(self.activityIndicator)
        
        DataManager.getAllMyCollection() { (myStoredCollections: [MyCollection]?) -> (Void) in
            
            if (myStoredCollections != nil) {
                self.myCollectionsArray = myStoredCollections!
                self.myCollectionsArray.sortInPlace({ $0.name < $1.name })

                self.tableView.reloadData()
                ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
            } else {
                ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Collections list cannot be loaded", comment: ""), view: self)
            }
        }
    }
    
    func deleteCollection(indexPath: NSIndexPath) {
        ActivityIndicatorHelper.startActivityIndicator(self.activityIndicator)
        
        let rowNumber = indexPath.row
        let deletedCollection = myCollectionsArray[rowNumber]
        
        DataManager.deleteMyCollection(deletedCollection, completion: { (allOk: Bool) -> (Void) in
            
            if allOk {
                self.myCollectionsArray.removeAtIndex(rowNumber)
                self.tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
                ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
            } else {
                ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Collection cannot be deleted", comment: ""), view: self)
            }
        })
    }
    
    func editCollection(indexPath: NSIndexPath) {
        let rowNumber = indexPath.row
        self.myCollectionToEdit = myCollectionsArray[rowNumber]
        
        self.performSegueWithIdentifier(SEGUE_FROM_MYCOLLECTIONS_TO_EDIT_NEWCOLLECTION, sender: self)
    }
    
    func addNewCollection() {
        self.performSegueWithIdentifier(SEGUE_FROM_MYCOLLECTIONS_TO_NEWCOLLECTION, sender: self)
    }
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if (segue.identifier == SEGUE_FROM_MYCOLLECTIONS_TO_EDIT_NEWCOLLECTION) {
            if let destinationVC = segue.destinationViewController as? CollectionsNewCollectionTableViewController {
                destinationVC.myCollection = self.myCollectionToEdit
            }
        } else if (segue.identifier == SEGUE_FROM_MYCOLLECTIONS_TO_ITEMLIST) {
            if let destinationVC = segue.destinationViewController as? ItemsListViewController {
                if let index = tableView.indexPathForSelectedRow?.row {
                    destinationVC.thisCollection = self.myCollectionsArray[index]
                }
            }
        }
    }
    
}
