//
//  CollectionsNewCollectionTableViewController.swift
//  CollectMe
//
//  Created by Oscar Fernandez Ardanuy on 15/04/16.
//  Copyright © 2016 dorhus. All rights reserved.
//

import UIKit

class CollectionsNewCollectionTableViewController: UITableViewController, UITextFieldDelegate {

    // layout objects
    @IBOutlet weak var nameTextField: UITextField!
    @IBOutlet weak var classTextField: UITextField!
    @IBOutlet weak var isCompleteSegmentedControl: UISegmentedControl! //0:Complete - 1:Incomplete
    @IBOutlet weak var isCompleteImageView: UIImageView!
    
    @IBOutlet weak var colorPickerView: UIPickerView!
    
    
    @IBAction func doneNavButton(sender: AnyObject) {
        doneActionButton()
    }
    @IBAction func isCompleteValueChangedAction(sender: AnyObject) {
        setIsCompleteIcon()
    }
    
    var myCollection:MyCollection? = nil
    var selectedColor: String? = nil
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.colorPickerView.dataSource = self
        
        
        // Do any additional setup after loading the view.
        if myCollection != nil {
            nameTextField.text = myCollection!.name
            classTextField.text = myCollection!.info
            selectedColor = myCollection!.color
            setUpColorsPicker()
            
            if myCollection!.isFull() {
                isCompleteSegmentedControl.selectedSegmentIndex = 0
            }
            setIsCompleteIcon()
            // nav bar title
            self.title = NSLocalizedString("Edit collection", comment: "")
        } else {
            selectedColor = CollectionsColor.colors[0]
        }
        
        setSyle()
    }
    
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
        
        StyleHelper.addSingleLineUnderTextField(nameTextField)
        StyleHelper.addSingleLineUnderTextField(classTextField)
    }
    
    // to press enter in delegated layout text fields
    func textFieldShouldReturn(textField: UITextField) -> Bool {
        switch textField {
        case nameTextField:
            // go to info text field
            classTextField.becomeFirstResponder()
        case classTextField:
            // close keyboard
            textField.resignFirstResponder()
        default:
            // close keyboard
            textField.resignFirstResponder()
        }
        return false
    }
    
    func doneActionButton() {
        
        if checkData() {
            let nameCollection = nameTextField.text
            let infoCollection = classTextField.text
            let isComplete:Bool = isCompleteSegmentedControl.selectedSegmentIndex == 0
            let colColor: String = selectedColor!
            
            if (myCollection == nil) {
                // SAVE NEW COLLECTION
                DataManager.saveNewMyCollection(nameCollection!, info: infoCollection, full: isComplete, color: colColor, completion: { (allOk:Bool) -> (Void) in
                    if allOk {
                        // return back
                        self.navigationController?.popViewControllerAnimated(true)
                    } else {
                        MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Collection cannot be created", comment: ""), view: self)
                    }
                })
            } else {
                // EDIT COLLECTION
                myCollection!.name = nameCollection
                myCollection!.info = infoCollection
                myCollection!.setIsFull(isComplete)
                myCollection!.color = colColor
                
                DataManager.updateMyCollection(myCollection!, completion: { (allOk:Bool) -> (Void) in
                    if allOk {
                        // return back
                        self.navigationController?.popViewControllerAnimated(true)
                    } else {
                        MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Collection cannot be edited", comment: ""), view: self)
                    }
                })
            }
            
        } else {
            MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Name is missing", comment: ""), view: self)
        }
        
    }
    
    func checkData() -> Bool {
        return nameTextField.text != nil && !nameTextField.text!.isEmpty
    }
    
    func setIsCompleteIcon() {
        isCompleteImageView.image = StyleHelper.getImageForIsCompleted(isCompleteSegmentedControl.selectedSegmentIndex == 0)
    }
    
}
