//
//  ItemsListViewController.swift
//  CollectMe
//
//  Created by Oscar Fernandez Ardanuy on 25/04/16.
//  Copyright © 2016 dorhus. All rights reserved.
//

import UIKit
import MGSwipeTableCell

class ItemsListViewController: UIViewController, UITableViewDataSource, MGSwipeTableCellDelegate {

    // layout objects
    @IBOutlet weak var tableView: UITableView!
    @IBAction func addNavButton(sender: AnyObject) {
        addNewItem()
    }
    
    var thisCollection:MyCollection! = nil
    internal var itemsSortedArray:[MyItem] = []
    internal var allItemsSortedArray:[MyItem] = []
    private var itemToEdit:MyItem? = nil

    
    // Search
    let searchController = UISearchController(searchResultsController: nil)

    private var activityIndicator: UIActivityIndicatorView?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Do any additional setup after loading the view.
        switch Utils.getOrigin(self.navigationController!) {
        case MyTabBarOrigin.CollectionsList:
            self.title = thisCollection.name
        case MyTabBarOrigin.Wanted:
            self.title = NSLocalizedString("Wanted", comment: "")
        case MyTabBarOrigin.Search:
            self.title = NSLocalizedString("Search", comment: "")
            self.setupSearchBar()
        default:
            self.title = NSLocalizedString("Search", comment: "")
            self.setupSearchBar()
        }
        setSyle()
        activityIndicator = ActivityIndicatorHelper.getActivityIndicator(self.view)
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func viewWillAppear(animated: Bool) {
        switch Utils.getOrigin(self.navigationController!) {
        case MyTabBarOrigin.CollectionsList:
            self.itemsSortedArray = thisCollection.getItemsArray().sort({ $0.name_title < $1.name_title })
            
        case MyTabBarOrigin.Wanted:
            ActivityIndicatorHelper.startActivityIndicator(self.activityIndicator)
            
            DataManager.getWishList() { (wishList: [MyItem]?) -> (Void) in
                
                if (wishList != nil) {
                    self.itemsSortedArray = wishList!.sort({ $0.name_title < $1.name_title })
                    ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                } else {
                    ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                    MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Wanted items list cannot be loaded", comment: ""), view: self)
                }
            }
            
        case MyTabBarOrigin.Search:
            ActivityIndicatorHelper.startActivityIndicator(self.activityIndicator)
            
            DataManager.getAllItems({ (allItemsArray: [MyItem]?) -> (Void) in
                
                if (allItemsArray != nil) {
                    self.allItemsSortedArray = allItemsArray!.sort({ $0.name_title < $1.name_title })
                    self.updateSearchResultsForSearchController(self.searchController)
                    ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                } else {
                    ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                    MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Items list cannot be loaded", comment: ""), view: self)
                }
            })
        default:
            break
            
        }
        
        self.tableView.reloadData()
        
    }
    
    
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */

    
    // MARK: UITableViewDataSource
    func tableView(tableView: UITableView,
        numberOfRowsInSection section: Int) -> Int {
            return self.itemsSortedArray.count

    }
    
    func tableView(tableView: UITableView,
        cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        
        //let cell = tableView.dequeueReusableCellWithIdentifier("MyCollectionCell", forIndexPath: indexPath) as! CollectionsListTableViewCell
        let cell = tableView.dequeueReusableCellWithIdentifier("MyItemCell", forIndexPath: indexPath) as! ItemsListMGSwipeTableCell
        let rowNumber = indexPath.row
        let cellItem = itemsSortedArray[rowNumber]
        
        cell.nameLabel.text = cellItem.name_title
        cell.descriptionLabel.text = cellItem.desc
    
        cell.ratingImageView.image = StyleHelper.getImageForRating(cellItem.personal_value as! Int)
        
        if cellItem.isWanted() {
            cell.iconWantedImageView.image = StyleHelper.getImageWantedForItemList()
        } else {
            cell.iconWantedImageView.image = nil
        }
        
        if let iconImage: NSData = cellItem.getBestImageThumbnail() {
            if let iconUIImage = Utils.NSDataToUIImage(iconImage) {
                cell.iconImageView.image = iconUIImage
            }
        } else {
            cell.iconImageView.image = StyleHelper.getImageForNoImageItemList()
        }
        
        // SWIPE functions
        let deleteButton = MGSwipeButton(title: NSLocalizedString("Delete", comment: ""), backgroundColor: UIColor.redColor())
        let editButton = MGSwipeButton(title: NSLocalizedString("Edit", comment: ""),backgroundColor: UIColor.lightGrayColor())
        
        cell.rightButtons = [deleteButton, editButton]
        cell.rightSwipeSettings.transition = MGSwipeTransition.Border

        cell.delegate = self
        
        return cell
    }
    // END: UITableViewDataSource
    
    
    //MARK: MGSwipeTableCellDelegate
    func swipeTableCell(cell:MGSwipeTableCell, index:Int,  direction:MGSwipeDirection, fromExpansion:Bool) -> Bool {
        //print("swipeTableCell")
        return true
    }
    
    func swipeTableCell(cell: MGSwipeTableCell!, tappedButtonAtIndex index: Int, direction: MGSwipeDirection, fromExpansion: Bool) -> Bool {
        let indexPath = tableView.indexPathForCell(cell)
        
        // index is the possition of the button inside the MGSwipeTableCell
        switch index {
        case 0: // DELETE
            deleteItem(indexPath!)
            //print("delete")
        case 1: // EDIT
            editItem(indexPath!)
            //print("edit")
        default:
            break
        }
        
        return true
    }
    //END: MGSwipeTableCellDelegate
 
    private func editItem(indexPath: NSIndexPath) {
        let rowNumber = indexPath.row
        self.itemToEdit = self.itemsSortedArray[rowNumber]
        
        self.performSegueWithIdentifier(SEGUE_FROM_ITEMLIST_TO_EDIT_NEWITEM, sender: self)
    }
    
    private func deleteItem(indexPath: NSIndexPath) {
        let rowNumber = indexPath.row
        let deletedItem = self.itemsSortedArray[rowNumber]
        
        ActivityIndicatorHelper.startActivityIndicator(self.activityIndicator)
        
        DataManager.deleteMyItem(deletedItem) { (allOk:Bool) -> (Void) in
            
            if allOk {
                self.itemsSortedArray.removeAtIndex(rowNumber)
                if (!self.allItemsSortedArray.isEmpty) {
                    self.allItemsSortedArray.removeAtIndex(self.allItemsSortedArray.indexOf(deletedItem)!)
                }
                self.tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
                ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
            } else {
                ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Item cannot be deleted", comment: ""), view: self)
            }
        }
    }
    
    func addNewItem() {
        self.performSegueWithIdentifier(SEGUE_FROM_ITEMLIST_TO_NEWITEM, sender: self)
    }
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if (segue.identifier == SEGUE_FROM_ITEMLIST_TO_NEWITEM) {
            if let destinationVC = segue.destinationViewController as? ItemsNewItemTableViewController {
                destinationVC.thisCollection = self.thisCollection
                destinationVC.newItemPage = true
            }
        } else if (segue.identifier == SEGUE_FROM_ITEMLIST_TO_EDIT_NEWITEM) {
            if let destinationVC = segue.destinationViewController as? ItemsNewItemTableViewController {
                destinationVC.thisItem = self.itemToEdit
                destinationVC.thisCollection = self.thisCollection
                destinationVC.editPage = true
            }
        } else if (segue.identifier == SEGUE_FROM_ITEMLIST_TO_ITEM_PAGE) {
            if let destinationVC = segue.destinationViewController as? ItemsNewItemTableViewController {
                if let index = tableView.indexPathForSelectedRow?.row {
                    destinationVC.thisItem = self.itemsSortedArray[index]
                    destinationVC.thisCollection = self.thisCollection
                    destinationVC.infoPage = true
                }
            }
        }
    }
}
