//
//  ItemsNewItemTableViewController.swift
//  CollectMe
//
//  Created by Oscar Fernandez Ardanuy on 25/04/16.
//  Copyright © 2016 dorhus. All rights reserved.
//

import UIKit

class ItemsNewItemTableViewController: UITableViewController, BarcodeProtocol {

    // layout objects
    // fields
    @IBOutlet weak var nameTextField: UITextField!
    @IBOutlet weak var typeTextField: UITextField!
    @IBOutlet weak var storedInTextField: UITextField!
    @IBOutlet weak var authorTextField: UITextField!
    @IBOutlet weak var shopTextField: UITextField!
    @IBOutlet weak var amountPaidTextField: UITextField!
    @IBOutlet weak var currencyPaidTextField: UITextField!
    @IBOutlet weak var serialOrderTextField: UITextField!
    @IBOutlet weak var serialSizeTextField: UITextField!
    // text views
    @IBOutlet weak var descriptionTextView: UITextView!
    @IBOutlet weak var notesTextView: UITextView!
    // text view buttons
    @IBAction func descriptionButtonAction(sender: AnyObject) {
        hideKeyboard(descriptionTextView)
    }
    @IBAction func notesButtonAction(sender: AnyObject) {
        hideKeyboard(notesTextView)
    }
    // segmented controls
    @IBOutlet weak var wantedSegmentedControl: UISegmentedControl! //0:I have it - 1: I want it
    @IBOutlet weak var isCompleteSegmentedControl: UISegmentedControl! //0:Complete - 1: Incomplete
    @IBOutlet weak var boxedSegmentedControl: UISegmentedControl! //0:Boxed - 1:Unboxed
    // rating control views
    //personalValueControlView
    //conditionControlView
    @IBOutlet weak var personalValueControlView: BlueStarsRatingControlView!
    @IBOutlet weak var conditionControlView: BlueStarsRatingControlView!
    // collection picker
    @IBOutlet weak var collectionPickerView: UIPickerView!
    // date picker
    @IBOutlet weak var releaseDatePicker: UIDatePicker!
    // buttons
    @IBOutlet weak var doneNavButton: UIBarButtonItem!
    @IBAction func doneNavButtonAction(sender: AnyObject) {
        doneActionButton()
    }
    @IBAction func capturePictureButton(sender: AnyObject) {
        addNewPicture()
    }
    //action
    @IBAction func wantedValueChanedAction(sender: AnyObject) {
        setWantedIcon()
    }
    //images
    @IBOutlet weak var wantedImageView: UIImageView!
    @IBOutlet weak var picturesScrollView: UIScrollView!
    
    
    var thisCollection:MyCollection! = nil
    var thisItem:MyItem? = nil
    var newItemPage: Bool = false
    var editPage:Bool = false // requires thisItem
    var infoPage:Bool = false // requires thisItem
    
    // collection picker
    var allCollections:[MyCollection] = []
    var nameNewCollection: String?
    var classNewCollection: String?
    
    // image picker for capture / load
    internal var imagePicker : UIImagePickerController?
    internal var picturesArray : CarouselPictures = CarouselPictures()
    
    // carousel
    var imagePressActionActive: Bool = false
    var pictureSelected: UIImage? = nil
    
    // barcode scanner
    @IBOutlet weak var barcodeTextField: UITextField!
    @IBAction func barcodeScannerButtonAction(sender: AnyObject) {
        openBarcodeScanner()
    }
    @IBOutlet weak var barcodeScannerButton: UIButton!
    
    
    internal var activityIndicator: UIActivityIndicatorView?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Uncomment the following line to preserve selection between presentations
        // self.clearsSelectionOnViewWillAppear = false

        // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
        // self.navigationItem.rightBarButtonItem = self.editButtonItem()
        
        // Collection selector
        self.collectionPickerView.dataSource = self
        self.collectionPickerView.delegate = self
        
        // Carousel
        setUpImagePicker()
        // nav button
        setNavButton()
        
        activityIndicator = ActivityIndicatorHelper.getActivityIndicator(self.view)
        
        if self.editPage || self.infoPage { // edit page or info page
            
            // If the origin is not collections list, find the collection's item
            if (Utils.getOrigin(self.navigationController) != MyTabBarOrigin.CollectionsList) {
                ActivityIndicatorHelper.startActivityIndicator(self.activityIndicator)
                
                DataManager.getMyCollection(from: self.thisItem!, completion: { (itemCollection: MyCollection?) -> (Void) in
                    ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                    
                    self.thisCollection = itemCollection
                    self.setUpCollectionsPicker()
                })
            } else {
                setUpCollectionsPicker()
            }
            
            // Pictures
            picturesArray = CarouselPictures.init(picturesArray: thisItem!.getImages())
            
            completeViewsWithItemData()
            
            // EDIT PAGE
            if self.editPage {
                self.title = NSLocalizedString("Edit item", comment: "")
                
            // INFO PAGE
            } else {
                self.title = NSLocalizedString("Item", comment: "")
                
                setNavButton()
                
                enableEditViews(false)
                if (thisItem!.desc == nil || thisItem!.desc!.isEmpty) {
                    
                }
            }
            
        // at new item page in wish list, select wanted at wanted segmented control
        } else if (Utils.getOrigin(self.navigationController) == MyTabBarOrigin.Wanted) {
            setUpCollectionsPicker()
            wantedSegmentedControl.selectedSegmentIndex = 1
        // new item
        } else {
            setUpCollectionsPicker()
        }
        
        setSyle()
    }
    
    override func viewDidLayoutSubviews() {
        StyleHelper.addSingleLineUnderTextField(self.nameTextField)
        StyleHelper.addSingleLineUnderTextField(self.typeTextField)
        StyleHelper.addSingleLineUnderTextField(self.storedInTextField)
        StyleHelper.addSingleLineUnderTextField(self.authorTextField)
        StyleHelper.addSingleLineUnderTextField(self.shopTextField)
        StyleHelper.addSingleLineUnderTextField(self.amountPaidTextField)
        StyleHelper.addSingleLineUnderTextField(self.currencyPaidTextField)
        StyleHelper.addSingleLineUnderTextField(self.serialOrderTextField)
        StyleHelper.addSingleLineUnderTextField(self.serialSizeTextField)
        StyleHelper.addSingleLineUnderTextField(self.barcodeTextField)
        
        StyleHelper.addBorderToTextView(self.descriptionTextView)
        StyleHelper.addBorderToTextView(self.notesTextView)
        
        self.printImagesInScrollView()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    private func doneActionButton() {
        // INFO PAGE
        if infoPage {
            enableEditItem()
        // EDIT PAGE or NEW ITEM PAGE
        } else {
            if checkData() {
                // + New collection seletec in the picker
                if self.thisCollection == nil {
                    showCreateNewCollectionAlert()
                    // Existing collection in the picker
                } else {
                    self.addOrEditItem()
                }
            } else {
                MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Name is missing", comment: ""), view: self)
            }
        }
    }
    
    // only for edit and info page
    private func completeViewsWithItemData() {
        nameTextField.text = thisItem!.name_title
        typeTextField.text = thisItem!.type_class
        storedInTextField.text = thisItem!.where_is
        authorTextField.text = thisItem!.author
        shopTextField.text = thisItem!.shop
        amountPaidTextField.text = thisItem!.amount_paid
        currencyPaidTextField.text = thisItem!.currency_paid
        barcodeTextField.text = thisItem!.barcode
        
        serialOrderTextField.text = thisItem!.serial_order
        serialSizeTextField.text = thisItem!.serial_size
        
        descriptionTextView.text = thisItem!.desc
        notesTextView.text = thisItem!.notes
        
        if (thisItem!.status_condition != nil) {
            conditionControlView.rating = thisItem!.status_condition as! Int
        }
        if (thisItem!.personal_value != nil) {
            personalValueControlView.rating = thisItem!.personal_value as! Int
        }
        
        // I have it / I want it selector
        if thisItem!.isWanted() {
            wantedSegmentedControl.selectedSegmentIndex = 1
        } else {
            wantedSegmentedControl.selectedSegmentIndex = 0
        }
        setWantedIcon()
        
        // Complete / incomplete selector
        if thisItem!.isComplete() {
            isCompleteSegmentedControl.selectedSegmentIndex = 0
        } else {
            isCompleteSegmentedControl.selectedSegmentIndex = 1
        }
        
        // Boxed / Unboxed selector
        if thisItem!.hasBox() {
            boxedSegmentedControl.selectedSegmentIndex = 0
        } else {
            boxedSegmentedControl.selectedSegmentIndex = 1
        }
        
        if (thisItem!.release_date != nil) {
            releaseDatePicker.date = thisItem!.release_date!
        }
    }
    
    private func enableEditViews(enable: Bool = true) {
        nameTextField.enabled = enable
        typeTextField.enabled = enable
        storedInTextField.enabled = enable
        authorTextField.enabled = enable
        shopTextField.enabled = enable
        amountPaidTextField.enabled = enable
        currencyPaidTextField.enabled = enable
        
        barcodeTextField.enabled = enable
        barcodeScannerButton.userInteractionEnabled = enable
        
        serialOrderTextField.enabled = enable
        serialSizeTextField.enabled = enable
        
        descriptionTextView.editable = enable
        descriptionTextView.selectable = enable
        notesTextView.editable = enable
        notesTextView.selectable = enable
        
        conditionControlView.userInteractionEnabled = enable
        personalValueControlView.userInteractionEnabled = enable
        
        wantedSegmentedControl.userInteractionEnabled = enable
        isCompleteSegmentedControl.userInteractionEnabled = enable
        boxedSegmentedControl.userInteractionEnabled = enable
        releaseDatePicker.userInteractionEnabled = enable
        collectionPickerView.userInteractionEnabled = enable
    }
    
    private func enableEditItem() {
        enableEditViews()
        editPage = true
        infoPage = false
        newItemPage = false
        self.title = NSLocalizedString("Edit item", comment: "")
        setNavButton()
    }
    
    private func setNavButton() {
        if (infoPage) {
            doneNavButton.title = NSLocalizedString("Edit", comment: "")
        } else {
            doneNavButton.title = NSLocalizedString("Done", comment: "")
        }
        
    }
    
    private func checkData() -> Bool {
        return nameTextField.text != nil && !nameTextField.text!.isEmpty
    }
    
    internal func addOrEditItem() {
        ActivityIndicatorHelper.startActivityIndicator(self.activityIndicator)
        
        DataManager.collectionExists(self.thisCollection, completion: { (collectionExists: Bool?) -> (Void) in
            
            // COLLECTION EXISTS
            if collectionExists != nil && collectionExists! {
                let name = self.nameTextField.text!
                let type = self.typeTextField.text
                let storedIn = self.storedInTextField.text
                let author = self.authorTextField.text
                let shop = self.shopTextField.text
                let description = self.descriptionTextView.text
                let notes = self.notesTextView.text
                let amountPaid = self.amountPaidTextField.text
                let currencyPaid = self.currencyPaidTextField.text
                let serialOrder = self.serialOrderTextField.text
                let serialSize = self.serialSizeTextField.text
                let barcode = self.barcodeTextField.text
                
                let wanted = self.wantedSegmentedControl.selectedSegmentIndex == 1
                let isComplete = self.isCompleteSegmentedControl.selectedSegmentIndex == 0
                let hasBox = self.boxedSegmentedControl.selectedSegmentIndex == 0
                
                let condition = self.conditionControlView.rating
                let personalValue = self.personalValueControlView.rating
                
                let releaseDate = self.releaseDatePicker.date
                
                // NEW ITEM PAGE
                if (self.newItemPage) {
                    
                    DataManager.saveNewMyItem(inside: self.thisCollection, name: name, desc: description,
                        notes: notes, barcode: barcode, shop: shop, type: type,
                        whereIs: storedIn, author: author, currencyPaid: currencyPaid,
                        amountPaid: amountPaid, condition: condition, personalValue: personalValue,
                        serialOrder: serialOrder, serialSize: serialSize, isComplete: isComplete,
                        isWanted: wanted, withBox: hasBox, releaseDate: releaseDate, images: self.picturesArray,
                        completion: { (allOk:Bool) -> (Void) in
                            ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                            
                            if allOk {
                                // return back
                                self.navigationController?.popViewControllerAnimated(true)
                            } else {
                                MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Item cannot be created", comment: ""), view: self)
                            }
                    })
                    
                // EDIT PAGE
                } else if (self.editPage) {
                    DataManager.itemExists(self.thisItem!, completion: { (itemExists: Bool?) -> (Void) in
                        
                        // ITEM EXISTS
                        if itemExists != nil && itemExists! {
                            self.thisItem!.name_title = name
                            self.thisItem!.type_class = type
                            self.thisItem!.where_is = storedIn
                            self.thisItem!.author = author
                            self.thisItem!.shop = shop
                            self.thisItem!.desc = description
                            self.thisItem!.notes = notes
                            self.thisItem!.amount_paid = amountPaid
                            self.thisItem!.currency_paid = currencyPaid
                            self.thisItem!.serial_order = serialOrder
                            self.thisItem!.serial_size = serialSize
                            self.thisItem!.barcode = barcode
                            
                            self.thisItem!.setIsWanted(wanted)
                            self.thisItem!.setIsComplete(isComplete)
                            self.thisItem!.setHasBox(hasBox)
                            
                            self.thisItem!.status_condition = condition
                            self.thisItem!.personal_value = personalValue
                            
                            self.thisItem!.release_date = releaseDate
                            
                            self.thisItem!.collection_node = self.thisCollection
                            
                            // images
                            
                            DataManager.updateMyItem(this: self.thisItem!, images: self.picturesArray, completion: { (allOk:Bool) -> (Void) in
                                ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                                
                                if allOk {
                                    // return back
                                    self.navigationController?.popViewControllerAnimated(true)
                                } else {
                                    MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Item cannot be edited", comment: ""), view: self)
                                }
                            })
                            // ITEM DOES NOT EXISTS, deleted before editing action
                        } else {
                            ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                            MessageHelper.showBasicAlertWithReturnBackNavigation(nil, message: NSLocalizedString("Item does not exist", comment: ""), view: self)
                        }
                    })
                } else {
                    ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                }
            // COLLECTION DOES NOT EXISTS, deleted before editing action
            } else {
                ActivityIndicatorHelper.stopActivityIndicator(self.activityIndicator)
                
                MessageHelper.showBasicAlertWithReturnBackNavigation(nil, message: NSLocalizedString("Collection does not exist", comment: ""), view: self)
            }
         })
    }
    
    private func setWantedIcon() {
        if (wantedSegmentedControl.selectedSegmentIndex == 1) {
            wantedImageView.image = StyleHelper.getImageWantedForItemList()
        } else {
            wantedImageView.image = nil
        }
    }
    
    private func addNewPicture() {
        if !infoPage {
            selectImageOriginOptionSheet()
        }
    }
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if (segue.identifier == SEGUE_FROM_ITEM_PAGE_TO_PICTURE_PAGE) {
            if let destinationVC = segue.destinationViewController as? PictureViewController {
                destinationVC.image = self.pictureSelected
            }
        } else if (segue.identifier == SEGUE_FROM_ITEM_PAGE_TO_BARCODE_SCANNER_PAGE) {
            if let destinationVC = segue.destinationViewController as? BarcodeScannerViewController {
                destinationVC.delegate = self
            }
        }
    }
    
    // BARCODE SCANNER
    
    private func openBarcodeScanner() {
        if (UIImagePickerController.isSourceTypeAvailable(.Camera)) {
            self.performSegueWithIdentifier(SEGUE_FROM_ITEM_PAGE_TO_BARCODE_SCANNER_PAGE, sender: self)
        } else {
            MessageHelper.showBasicAlert(nil, message: NSLocalizedString("Only available in devices with camera", comment: ""), view: self)
        }
        
    }
    
    func setResultOfBarcodeScanner(valueSent: String) {
        self.barcodeTextField.text = valueSent
        //print("ItemsNewItemTableViewController.setResultOfBarcodeScanner: " + self.barcodeData!)
    }
}
