//
//  RatingControlView.swift
//  CollectMe
//
//  Created by Oscar Fernandez Ardanuy on 27/04/16.
//  Copyright © 2016 dorhus. All rights reserved.
//

import UIKit

class RatingControlView: UIView {

    // MARK: constants
    let WIDTH = 42
    let HEIGHT = 40
    let STARS = 5
    let SPACE = 5 // space width

    // MARK: Properties
    
    var rating:Int = 0 {
        didSet {
            setNeedsLayout()
        }
    }
    var ratingButtons = [UIButton]()
    
    
    // MARK: Initialization
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        
        let filledStarImage = UIImage(named: getStarFilledImageName())
        let emptyStarImage = UIImage(named: getStarEmptyImageName())
        
        for _ in 0..<self.STARS {
            let button = UIButton()
            button.setImage(emptyStarImage, forState: .Normal)
            button.setImage(filledStarImage, forState: .Selected)
            button.setImage(filledStarImage, forState: [.Highlighted, .Selected])
            button.adjustsImageWhenHighlighted = false
            button.addTarget(self, action: #selector(RatingControlView.ratingButtonTapped(_:)), forControlEvents: .TouchDown)

            ratingButtons.append(button)
            addSubview(button)
        }
    }
    
    override func layoutSubviews() {
        var buttonFrame = CGRect(x: 0, y: 0, width: self.WIDTH, height: self.HEIGHT)
        
        for (index, button) in ratingButtons.enumerate() {
            buttonFrame.origin.x = CGFloat(index * (self.WIDTH + self.SPACE))
            button.frame = buttonFrame
        }
        
        updateButtonSelectionStates()
    }

    
    // MARK: image names
    
    func getStarFilledImageName() -> String {
        return "star_filled"
    }
    
    func getStarEmptyImageName() -> String {
        return "star_empty"
    }
    
    
    // MARK: Button Action
    
    func ratingButtonTapped(button: UIButton) {
        if (ratingButtons.indexOf(button) == 0 && rating == 1) {
            rating = 0
        } else {
            rating = ratingButtons.indexOf(button)! + 1
        }
    
        updateButtonSelectionStates()
    }
    
    
    override func intrinsicContentSize() -> CGSize {
        let width = (self.WIDTH * self.STARS) + (self.SPACE * (self.STARS - 1))
        
        return CGSize(width: width, height: self.HEIGHT)
    }
    
    func updateButtonSelectionStates() {
        for (index, button) in ratingButtons.enumerate() {
            button.selected = index < rating
        }
    }
}
