//
//  StyleHelper.swift
//  CollectMe
//
//  Created by Oscar Fernandez Ardanuy on 25/04/16.
//  Copyright © 2016 dorhus. All rights reserved.
//

import Foundation
import UIKit

class StyleHelper {
    
    static func getTitleFont() -> UIFont {
        return UIFont(name: "AppleGothic", size: 20)!
    }
    
    static func setNavigationBarTitleStyle(view: UIViewController) {
        view.navigationController?.navigationBar.titleTextAttributes = [ NSFontAttributeName: StyleHelper.getTitleFont()]
        view.navigationController?.navigationBar.barStyle = UIBarStyle.Black
        view.navigationController?.navigationBar.tintColor = UIColor.whiteColor()
        view.navigationController?.navigationBar.barTintColor = UIColor(red: 0.0, green: 0.0, blue: 0.80, alpha: 1.0)
    }
    
    // COLORS
    static func getCellColor(color: String?) -> UIColor {
        
        var cellColor = color
        if color == nil {
            cellColor = "white"
        }
        
        switch cellColor! {
        case "red":
            return UIColor(red: 1.0, green: 0.0, blue: 0.0, alpha: 0.2)
        case "green":
            return UIColor(red: 0.0, green: 1.0, blue: 0.0, alpha: 0.2)
        case "blue":
            return UIColor(red: 0.0, green: 0.0, blue: 1.0, alpha: 0.2)
        case "yellow":
            return UIColor(red: 1.0, green: 1.0, blue: 0.0, alpha: 0.2)
        case "pink":
            return UIColor(red: 1.0, green: 0.4, blue: 0.7, alpha: 0.2)
        case "white":
            return UIColor.whiteColor()
        case "black":
            return UIColor(red: 0.0, green: 0.0, blue: 0.0, alpha: 0.2)
        default:
            return UIColor.whiteColor()
        }
    }
    
    // BORDERS
    static func addBorderToTextView(textView: UITextView) {
        let borderColor : UIColor = UIColor(red: 0.85, green: 0.85, blue: 0.85, alpha: 0.85)
        textView.layer.borderWidth = 0.5
        textView.layer.borderColor = borderColor.CGColor
        textView.layer.cornerRadius = 5.0
    }
    
    static func addSingleLineUnderTextView(textView: UITextView) {
        let border = CALayer()
        let width = CGFloat(0.5)
        border.borderColor = UIColor(red: 0.85, green: 0.85, blue: 0.85, alpha: 0.85).CGColor
        border.frame = CGRect(x: 0, y: textView.frame.size.height - width,
            width: textView.frame.size.width, height: textView.frame.size.height)
        
        border.borderWidth = width
        textView.layer.addSublayer(border)
        textView.layer.masksToBounds = true
    }
    
    static func addSingleLineUnderTextField(textField: UITextField) {
        let border = CALayer()
        let width = CGFloat(0.5)
        border.borderColor = UIColor(red: 0.85, green: 0.85, blue: 0.85, alpha: 0.85).CGColor
        border.frame = CGRect(x: 0, y: textField.frame.size.height - width,
            width: textField.frame.size.width, height: textField.frame.size.height)
        
        border.borderWidth = width
        textField.layer.addSublayer(border)
        textField.layer.masksToBounds = true
    }
    
    // IMAGES
    static func getImageForRating(rating:Int) -> UIImage? {
        let imageName = "\(rating)Stars"
        return UIImage(named: imageName)
    }
    
    static func getImageWantedForItemList() -> UIImage? {
        return UIImage(named: "red_heart")
    }
    
    static func getImageForIsCompleted(isComplted:Bool) -> UIImage? {
        if isComplted {
            return UIImage(named: "icon_half_full")
        } else {
            return UIImage(named: "icon_half_empty")
        }
    }
    
    static func getImageForNoImageItemList() -> UIImage? {
        return nil
    }

}
